

pro pingsoft_core, infile, pings, core, ps_file, x0, y0, WIDGET=widget, LOAD=load, $
                   PT=pt, EXTENSION=extension, PREFIX=prefix, SHIFT_PT=shift_PT, $
                   MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
                   FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
                   CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                   SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                   LARGE=large, NOEXTRACT=noextract, INVERSE=inverse, _EXTRA=extra


if keyword_set(widget) and not keyword_set(load) then goto, widget_on

if keyword_set(load) then begin
   windex1 = load.windex1
   windex2 = load.windex2
   widget = 1
endif

;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif

;--------------------------  Strings

tmp0 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
tmp1 = strsplit(tmp0,'.rscube.fits.gz',/extract,/regex)
tmp2 = strsplit(tmp1,'.rscube.fits',/extract,/regex)
tmp3 = strsplit(tmp2,'.rss.fits.gz',/extract,/regex)
tmp4 = strsplit(tmp3,'.rss.fits',/extract,/regex)
tmp5 = strsplit(tmp4,'.fits.gz',/extract,/regex)
tmp6 = strsplit(tmp5,'.fits',/extract,/regex)
object = tmp6

;--------------------------  Reading data

fits = readfits(infile,hdr,/silent)

hdr_obj = sctrim(fxpar(hdr,'OBJECT'))
if hdr_obj ne '0' then object = hdr_obj

author = fxpar(hdr,'AUTHOR')
author = string(author)

if keyword_set(extension) then begin
   if author eq 'CALIFA Collaboration' then $
      fits = readfits(infile,ext=extension,/silent) else $
      fits = readfits(infile,hdr,ext=extension,/silent)
endif

size = size(fits)
type = size[0]

;-------------------------- Testing for CALIFA data

califa = 0
ext_bpix = 0
if keyword_set(extension) then goto, ext_set

extend = fxpar(hdr,'EXTEND')

if author eq 'CALIFA Collaboration' then begin

   califa = 1

   ; CALIFA intensity map
   if not keyword_set(ct) then ct = 42

   ; CALIFA header without extensions
   if extend eq 0 then begin
      ext_bpix = -1
      goto, zero_ext
   endif

   ; Reading extensions
   if extend eq 1 then begin
      if type eq 2 then begin
         ext_bpix = readfits(infile,ext=2,/silent)
         ext_pt = readfits(infile,hdr_pt,ext=3,/silent)
      endif
      if type eq 3 then ext_bpix = readfits(infile,ext=3,/silent)
   endif

   zero_ext:
   ; Object type
   object = sctrim(fxpar(hdr,'OBJECT'))
   califaid = sctrim(fxpar(hdr,'CALIFAID'),format='(i4)')
   
endif

ext_set:

;--------------------------  Graphic device settings

title_str = object
if califa eq 1 then title_str = 'CALIFA: '+object else title_str = object

pingsoft_param, pings, title_str
pingsoft_display, pings, /silent


;--------------------------  Data type

; Assuming data in arcsec
mas = 0


; RSS type
if type eq 2 then begin

   pixels = size[1]
   rows   = size[2]

   crval = fxpar(hdr,'CRVAL1')
   cdelt = fxpar(hdr,'CDELT1')
   crpix = fxpar(hdr,'CRPIX1')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX1 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   ref_ra  = fxpar(hdr,'RA')
   ref_dec = fxpar(hdr,'DEC')

   data = fits

   if keyword_set(califa) and ext_bpix[0] ne -1 then begin
      ; Badpix map
      bpix = ext_bpix
      ; Position table
      xpt = tbget(hdr_pt, ext_pt, 1)
      ypt = tbget(hdr_pt, ext_pt, 2)
      shape = fxpar(hdr_pt,'FIBSHAPE')
      pt_sizex = fxpar(hdr_pt,'FIBSIZEX')
      pt_sizey = fxpar(hdr_pt,'FIBSIZEy')
      header = shape+string(pt_sizex)+string(pt_sizey)+'  0'
      xpt = -xpt
      openw,1,'tmp_3D.pt.txt'
      printf,1,header
      for i=0,n_elements(xpt)-1 do printf,1, string(i+1), xpt[i], ypt[i], 0
      close,1
      pt = 'tmp_3D.pt.txt'
   endif

   hdr1 = hdr
   hdr2 = hdr
   hdr3 = hdr

   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CTYPE1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr2, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

endif



; CUBE type
if type eq 3 then begin

   xx = size[1]
   yy = size[2]

   rows = xx*yy

   crval  = fxpar(hdr,'CRVAL3')
   cdelt  = fxpar(hdr,'CDELT3')
   crpix  = fxpar(hdr,'CRPIX3')
   pixels = fxpar(hdr,'NAXIS3')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX3 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   data = fltarr(pixels,rows)
   if keyword_set(califa) then bpix = fltarr(pixels,rows)

   ; New vector
   id = long(0)
   for j=0l, yy-1 do begin

      for i=0l, xx-1 do begin

         array = fits[i,j,*]
         array = transpose(array)

         ; NaN and Inf step 1
         mask = finite(array)
         array = array*mask
         nan = where(array ne array)
         if n_elements(nan) gt 1 then array[nan] = 0.
         if n_elements(nan) eq 1 and nan[0] ge 0 then array[nan] = 0.
         ; NaN and Inf step 2
         inf = where(array lt -1e10 or array gt 1e10)
         if n_elements(inf) gt 1 then array[inf] = 0.
         if n_elements(inf) eq 1 and inf[0] ge 0 then array[inf] = 0.
       
         data[*,id] = array
         
         if keyword_set(califa) and ext_bpix[0] ne -1 then begin
            array_bp = ext_bpix[i,j,*]
            array_bp = transpose(array_bp)
            bpix[*,id] = array_bp
         endif

         id = id+1

      endfor
      
   endfor

   cdelt1 = fxpar(hdr,'CDELT1')
   cdelt2 = fxpar(hdr,'CDELT2')
   crpix1 = fxpar(hdr,'CRPIX1')
   crpix2 = fxpar(hdr,'CRPIX2')
   naxis1 = fxpar(hdr,'NAXIS1')
   naxis2 = fxpar(hdr,'NAXIS2')

    RA = fxpar(hdr,'CRVAL1')
   Dec = fxpar(hdr,'CRVAL2')

   ref_ra  = RA
   ref_Dec = Dec

   ; Milli-arcseconds?
   if cdelt1 lt 1 or cdelt2 lt 1 then begin
      num = sctrim(cdelt1, format='(e)')
      exp = strsplit(num[0],'e',/extract,/regex)
      if exp[1] eq '-06' then begin
         cdelt1 = cdelt1 * 1e6
         cdelt2 = cdelt2 * 1e6
         mas = 1
      endif
   endif

   if cdelt1 eq 0 or cdelt2 eq 0 then resolution = 1 else resolution = cdelt1
   
   if crpix1 ne 0 then  RA_off = double(crpix1*resolution) else RA_off = 0
   if crpix2 ne 0 then Dec_off = double(crpix2*resolution) else Dec_off = 0
   
   if  RA_off eq 0 then  RA_off = xx*resolution*0.5
   if Dec_off eq 0 then Dec_off = yy*resolution*0.5

   rectangular_pt, xx, yy, resolution, 1, xpt, ypt, off, $
                   X_OFF=RA_off, Y_OFF=Dec_off, OUT_PT='tmp_3D.pt.txt' ;, /PPAK

   pt = 'tmp_3D.pt.txt'

   ; FITS header for RSS data and 1D FITS
   
   hdr1 = hdr
   hdr3 = hdr

  ; hdr1
   fxaddpar, hdr1, 'CRVAL1', crval, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CDELT1', cdelt, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRPIX1', crpix, ' updated by PINGSoft'
   sxdelpar, hdr1, 'CRVAL2'
   sxdelpar, hdr1, 'CRVAL3'
   sxdelpar, hdr1, 'CRPIX2'
   sxdelpar, hdr1, 'CRPIX3'
   sxdelpar, hdr1, 'CDELT2'
   sxdelpar, hdr1, 'CDELT3'
   sxdelpar, hdr1, 'DISPAXIS'
   sxdelpar, hdr1, 'WCSDIM'
   sxdelpar, hdr1, 'CTYPE1'
   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CD1_2'
   sxdelpar, hdr1, 'CD2_1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

   hdr2 = hdr1

endif


if cdelt eq 0 then begin
   print, ''
   print, ' WARNING: CDELT entry not found, using CDELT=1'
   print, ''
   cdelt = 1
endif

lambda = dblarr(pixels)
for i=0, pixels-1 do lambda[i] = (crval + i*cdelt)-(crpix-1)*cdelt

; Data in microns?
if mean(lambda) lt 100 then lambda = lambda*1e4

; Data in mas?
if mas eq 1 then wcs = 0

; Full lambda
lam = lambda

if not keyword_set(lmin) then lmin = min(lambda)

if not keyword_set(lmax) then lmax = max(lambda)

indice = where(lambda ge lmin and lambda le lmax)

lambda = lambda[indice]

xmin = min(lambda) & xmax = max(lambda)


widget_on:
if keyword_set(widget) and not keyword_set(load) then begin

   ; Variables from structure
   tag = get_tags(widget)
   for k=0, n_elements(tag)-1 do begin
      dumb = strmid(tag[k],1)
       com = execute(dumb+' = widget.'+dumb)
    endfor
    pingsoft_param, pings, title_str

endif

widget_param, wparam


;--------------------------  Broad/Narrow band 

if crval ne 0. then begin

    ; Pick filter
    if not keyword_set(filter) then filter = pings.filter
    pingsoft_filters, filter, sband, lband, fband, cwl

    bad_cwl:

    ; Shift
    if not keyword_set(band) then band = cwl
    shift = cwl/band
    shift = shift[0]
    newl = lband/shift
  
    ; Interpolation with data
    res = interpol(fband,newl,lam)
    nn = where(lam lt min(newl) or lam gt max(newl))
    if n_elements(nn) gt 1 then res[nn] = 0.

    mm = where(lam ge min(newl) and lam le max(newl))
    if n_elements(mm) eq 1 and mm[0] lt 0 then begin
       print, ''
       print, 'WARNING: Using a V-band filter within the data wavelength limits: '+sctrim(lmin)+' - '+sctrim(lmax)
       print, ''
       filter = 4
       pingsoft_filters, filter, sband, lband, fband, cwl
       band = mean(lam)
       goto, bad_cwl
    endif

    ; convolution
    convolve = res[mm]
    dband = fltarr(rows)
    for k=0l, rows-1 do dband[k] = total(convolve*data[mm,k])

    ; Negative values
    nband = where(dband lt 0)
    if nband[0] gt 0 then dband[nband]=0
    mband = min(dband)

    ;------ Scalings

    if keyword_set(clip) then begin
       scale = pingsoft_scl( dband, /scale, stretch=2, minvalue=min(dband), maxvalue=max(dband), negative=inverse )
       goto, scale_set
    endif
    if keyword_set(gamma) then begin
       scale = pingsoft_scl( dband, /scale, stretch=3, minvalue=min(dband), maxvalue=max(dband), negative=inverse )
       goto, scale_set
    endif
    if keyword_set(log) then begin
       scale = pingsoft_scl( dband, /scale, stretch=4, minvalue=min(dband), maxvalue=max(dband), negative=inverse )
       goto, scale_set
    endif
    if keyword_set(asinh) then begin
       scale = pingsoft_scl( dband, /scale, stretch=5, minvalue=min(dband), maxvalue=max(dband), beta=30, negative=inverse )
             goto, scale_set
    endif 
     if keyword_set(sqrt) then begin
        scale = pingsoft_scl( dband, /scale, stretch=6, minvalue=min(dband), maxvalue=max(dband), negative=inverse )
       goto, scale_set
    endif
    if keyword_set(histogram) then begin
       scale = pingsoft_scl( dband, /scale, stretch=7, minvalue=min(dband), maxvalue=max(dband), negative=inverse )
       goto, scale_set
    endif
    if keyword_set(gaussian) then begin
       scale = pingsoft_scl( dband, /scale, stretch=8, minvalue=min(dband), maxvalue=max(dband), negative=inverse )
       goto, scale_set
    endif

    ;------ Default scaling (linear, clipping=1%)
    scale = pingsoft_scl( dband, /scale, stretch=1, clip=1, negative=inverse)

    scale_set:

    ; Borders
    nband = where(dband gt mband and scale le 5)
    if nband[0] gt 0 then scale[nband] = 5

endif else begin
    print, ''
    print, 'ERROR: CRVAL1, CDELT1 and/or CRPIX1 not found in RSS header!'
    print, ''
    return
endelse


if keyword_set(widget) and not keyword_set(load) then goto, widget_on2 


;--------------------------  Position table

if keyword_set(pt) then begin

   pos_table = sctrim(pt)

   if file_test(pos_table) eq 0 then begin
      print, ''
      print, "ERROR: The file:  '"+pos_table+"'  does not exist!"
      print, ''
      return
   endif

endif


if not keyword_set(pt) then begin

   test_pt = strsplit(infile,'.fits',/extract,/regex)+'.pt.txt'

   if file_test(test_pt) eq 1 then pos_table = test_pt

   if file_test(test_pt) eq 0 then begin

      pos_table = identify_rss(rows,hdr) 

      if pos_table eq 'none' then begin
         print, ''
         print, "ERROR: PINGSoft cannot find/determine the associated position table"
         print, ''
         print, "       (is the instrument's resolution present in the FITS header?)"
         print, ''
         print, "       try setting the PT='pos_table.pt.txt' keyword"
         print, ''
         return
      endif
      
   endif

endif

pos_table = pos_table[0]


;-------------------------- Shifting PT to X/Y offsets

if keyword_set(shift_pt) then begin

   shift_ptable, pos_table, 'tmp.pt.txt', x0, y0, /silent

   pos_table = 'tmp.pt.txt'

endif


;-------------------------- Position table tests

; Mosaic type (reading first line of the position table)

readcol, pos_table, f='a,f,f,i', mtype, xsize, ysize, numline=1, /silent

xsize = xsize[0]
ysize = ysize[0]

if mtype eq 'C' then begin
    points = 50
    fiber_size = xsize*2
    radius = fiber_size/2
    ang = 2*!PI*findgen(points+1.)/points
    xarr = radius*cos(ang)
    yarr = radius*sin(ang)
endif

if mtype eq 'S' then begin
    radius = xsize*0.5
    xarr = [-1,1,1,-1,-1]*radius
    yarr = [-1,-1,1,1,-1]*radius
endif


; Read position table

readcol, pos_table, f='l,f,f,f', id, x, y, flag, /silent

if n_elements(id) ne rows then begin
   print, ''
   print, 'ERROR: Number of rows in RSS file not equal to number of entries in the position table'
   print, ''
   print, '       Are you using the right position table?'
   print, ''
   print, '       PT file: '+pos_table
   print, ''
   return
endif


;--------------------------  Fibers in mosaic

!p.multi=[0,1,1]

loadct, 1, /silent

; Spaxels window limits
box_limits = pingsoft_boxlim(id,x,y,radius)
xmax_mosaic = box_limits[0]
xmin_mosaic = box_limits[1]
ymax_mosaic = box_limits[2]
ymin_mosaic = box_limits[3]


; Windows position and dimensions

win_dim = pingsoft_win()

factor1=1.0 & factor2=1.0

if keyword_set(large) then begin
   factor1=1.5
   factor2=0.8
endif
xwin0_size = win_dim[0]*factor1
ywin0_size = win_dim[1]*factor1
xwin2_size = win_dim[2]*factor2
ywin2_size = win_dim[3]*factor2

widget_on2:

if keyword_set(ps) then begin
   
   if not keyword_set(font) then psfont=pings.font else psfont=font
   
   pingsoft_pson, ps_file, pings.x_ps, pings.y_ps, font=psfont, axis_thick=pings.axis, encapsulate=1
     
   if mas eq 0 then units = 'arcsec' else units = 'mas'

   plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
         xs=1, ys=1, charsize=pings.csize_ps, position=pings.pos0, $
         xtitle='!9D!X!e !nRA ('+units+')', ytitle='!9D!X!e !nDec ('+units+')', $
         title=title_str, _extra=extra
   goto, ps_on

endif


if keyword_set(plot) then begin

   if mas eq 0 then begin
      xtitle_w0 = pings.xtitle0
      ytitle_w0 = pings.ytitle0
   endif
   if mas eq 1 then begin
      xtitle_w0 = pings.xtitle3
      ytitle_w0 = pings.ytitle3
   endif

   set_font
   if keyword_set(load) then begin
      wset, windex2
      plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
            xs=1, ys=1, xthick=wparam.wthick, ythick=wparam.wthick, position=wparam.pos0, charsize=wparam.csize, $
            xtitle=xtitle_w0, ytitle=ytitle_w0
   endif
   if keyword_set(widget) then begin
      wset, windex2
      plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
            xs=1, ys=1, xthick=wparam.wthick, ythick=wparam.wthick, position=wparam.pos0, charsize=wparam.csize, $
            xtitle=xtitle_w0, ytitle=ytitle_w0, /noerase
      goto, ps_on
   endif
   window, 0, xs=xwin0_size, ys=ywin0_size
   plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
         xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, position=pings.pos0, charsize=pings.csize, $
         xtitle=xtitle_w0, ytitle=ytitle_w0, $
         title=pings.title0, /noerase, _extra=extra
endif

ps_on:

;--------- Color tables 

if not keyword_set(ct) then ct = 1

if ct eq 99 then ct = 0

spawn, 'echo $PINGSOFT_PATH', ppath
ctfile = ppath+'/pingsoft_ct.tbl'

ctables = [4,5,25,27,33,39,41,42]
ntest = where(ctables eq ct)
if ntest[0] ge 0 then full_ct = 1 else full_ct = 0


; Plot spaxels

if keyword_set(plot) or keyword_set(ps) then begin

   for i=0l, n_elements(x)-1 do begin
      
      xcoor =  x[i]
      ycoor =  y[i]
      
      x_fib = xarr + xcoor
      y_fib = yarr + ycoor

      loadct, ct, file=ctfile, /silent
      polyfill, x_fib, y_fib, color=scale[i]

      loadct, 1, /silent
      
      if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor1
      
      if keyword_set(ps) then begin
         loadct, ct, file=ctfile, /silent
         polyfill, x_fib, y_fib, color=scale[i]
         loadct, 1, /silent
         if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor2
      endif
      
   endfor

endif


if type eq 2 then begin

   ; Resolution
   ll = n_elements(x)
   testx = fltarr(ll)
   for i=0, ll-2 do testx[i] = abs(x[i+1]-x[i])
   testx = testx[where(testx gt 0.)]
   resol = min(testx)

   xx = ( max(x) - min(x) + resol ) / resol
   yy = ( max(y) - min(y) + resol ) / resol

   ; Geometry
   xgeom = xx-1
   if xgeom gt n_elements(x) then xgeom = n_elements(x)-1
   testx = x[0:xgeom]
   if min(testx) eq max(testx) then geometry = 2 else geometry = 1

   RA_off = xx*resol*0.5
   Dec_off = yy*resol*0.5
   resolution = resol

endif

fmap = 0

; Scale vector to 2D map (?)

if type eq 3 then begin

   fmap = fltarr(xx,yy)

   nn = where(fmap eq fmap)
   fmap[nn] = scale

endif


; Structure
if not keyword_set(califa) then bpix = 0
if ref_ra le 0 then wcs = 0 else wcs = 1

; widget
if type eq 2 then begin
   if wcs eq 0 then $
      hdr_coor = '  Spaxel ID     [RA,Dec] offset   (No WCS found in header)' else $
      hdr_coor = '  Spaxel ID     [RA,Dec] offset        RA              Dec           RA deg      Dec deg'
endif
if type eq 3 then begin
   if wcs eq 0 then $
      hdr_coor = '    Spaxel       ID      [RA,Dec] offset   (No WCS found in header)' else $
      hdr_coor = '    Spaxel       ID      [RA,Dec] offset         RA              Dec           RA deg      Dec deg'
endif

if not keyword_set(windex1) then windex1 = 2
if not keyword_set(windex2) then windex2 = 0

if not keyword_set(min_flux) then min_flux = 0.
if not keyword_set(max_flux) then max_flux = 0.

if not keyword_set(pt)   then pt = ''
if not keyword_set(plot) then plot = 1
if not keyword_set(draw) then draw = 0
if not keyword_set(ps)   then ps = 0
if not keyword_set(path) then path = ''

if not keyword_set(inverse) then inverse = 0
if not keyword_set(vline)   then vline = ''
if not keyword_set(region)  then region = 0

if not keyword_set(linear)   then linear = 0
if not keyword_set(clip)   then clip = 0
if not keyword_set(gamma)  then gamma = 0
if not keyword_set(log)    then log = 0
if not keyword_set(asinh)  then asinh = 0
if not keyword_set(histogram) then histogram = 0
if not keyword_set(gaussian)  then gaussian = 0
if not keyword_set(sqrt)  then sqrt = 0
if not keyword_set(stddev)  then stddev = 0

if ext_bpix[0] eq -1 then bpix=-1

core = {$
       infile: infile ,$
       data: data     ,$
       object:object  ,$
       ext_bpix:ext_bpix ,$
       bpix: bpix     ,$
       hdr: hdr       ,$
       hdr1: hdr1     ,$
       hdr2: hdr2     ,$
       hdr3: hdr3     ,$
       ref_RA: ref_RA ,$
       ref_Dec:ref_Dec,$
       pixels:pixels  ,$
       rows: rows     ,$
       lambda: lambda ,$
       pos_table: pos_table ,$
       type:  type    ,$
       mtype: mtype   ,$
       id: id ,$
       x: x   ,$
       y: y   ,$
       xx: xx   ,$
       yy: yy   ,$
       mas: mas ,$
       resolution: resolution  ,$
       RA_off:   RA_off ,$
       Dec_off: Dec_off ,$
       xarr: xarr  ,$
       yarr: yarr  ,$
       radius: radius   ,$
       scale: scale     ,$
       fmap: fmap       ,$
       win_dim: win_dim ,$
       xmax_mosaic: xmax_mosaic ,$
       xmin_mosaic: xmin_mosaic ,$
       ymax_mosaic: ymax_mosaic ,$
       ymin_mosaic: ymin_mosaic ,$
       xwin0_size:  xwin0_size  ,$
       ywin0_size:  ywin0_size  ,$
       xwin2_size:  xwin2_size  ,$
       ywin2_size:  ywin2_size  ,$ 
       indice:  indice ,$
       sband:   sband  ,$
       res:     res    ,$
       ; widget
       pt: pt      ,$
       path: path  ,$
       ct: ct      ,$
       lam: lam    ,$
       lmin: lmin  ,$
       lmax: lmax  ,$
       xmin: xmin  ,$
       xmax: xmax  ,$
       cwl: cwl    ,$
       wcs: wcs    ,$
       box_limits: box_limits  ,$
       windex1: windex1   ,$
       windex2: windex2   ,$
       title_str: title_str ,$
       crval: crval ,$
       plot: plot   ,$
       draw: draw   ,$
       linear: linear ,$
       clip:   clip   ,$
       asinh:  asinh  ,$
       gamma:  gamma  ,$
       log:    log    ,$
       stddev: stddev ,$
       sqrt:   sqrt   ,$
       histogram: histogram ,$
       gaussian:  gaussian  ,$
       hdr_coor: hdr_coor   ,$
       filter: filter   ,$
       inverse: inverse ,$
       califa: califa   ,$
       min_flux: min_flux  ,$
       max_flux: max_flux  ,$
       tmp0: tmp0   ,$
       ps: ps       ,$
       vline: vline ,$
       band: band   ,$
       xsize: xsize ,$
       ysize: ysize ,$
       region: region ,$

       eof:''$
}


end

