
;+
; NAME:
;       PINGSOFT_FILTERS
;
; PURPOSE:
;
;       Reads the default or user's supplied filters for PINGSoft.
;
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


pro plot_filters, PS=ps

    pingsoft_param, pings, ''
    pingsoft_display, pings, /silent
    cthick = 1

    set_font
    if ps eq 0 then window, 0, xs=900, ys=600

    if ps eq 1 then begin
       pingsoft_pson, 'PINGSoft_filters.eps', 24, 15, /encapsulate
       cthick = 2
    endif
    loadct, 0, /silent
    plot, [0], /nodata, xr=[2900,9900], yr=[0,1.15], xs=1, ys=1, $
          xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize*1.1, $
          xtitle='Wavelength', ytitle='Transmission', title='PINGSoft filters'

    spawn, 'echo $PINGSOFT_PATH', path
    ctfile = path+'/pingsoft_ct.tbl'
    loadct, 41, file=ctfile, /silent
    fpath = path+'/filters/'

    
    ; Halpha
    group = fpath+'ALL_Halpha.txt'
    readcol, group, f='a', file, /silent
    readcol, fpath+file[0], lband, fband, /silent
    oplot, lband, fband, color=174, thick=cthick
    readcol, fpath+file[1], lband, fband, /silent
    oplot, lband, fband, color=78, thick=cthick
    xyouts, 8800, 1.00, 'Halpha 20A FWHM',  align=0.5, charsize=pings.csize, color=174
    xyouts, 8800, 0.95, 'Halpha 80A FWHM',  align=0.5, charsize=pings.csize, color=78
    ;ang = ang()
    ;xyouts, 8800, 1.00, 'H'+latex('\alpha')+' 20'+ang+' FWHM',  align=0.5, charsize=pings.csize, color=174
    ;xyouts, 8800, 0.95, 'H'+latex('\alpha')+' 80'+ang+' FWHM',  align=0.5, charsize=pings.csize, color=78

    ; Johnson
    group = fpath+'ALL_Johnson.txt'
    readcol, group, f='a', file, /silent
    for i=0, n_elements(file)-1 do begin
       readcol, fpath+file[i], lband, fband, /silent
       oplot, lband, fband, color=128, thick=cthick
       xyouts, 8800, 0.90, 'Johnson (1965)',  align=0.5, charsize=pings.csize, color=128
    endfor
    ; Bessell
    group = fpath+'ALL_Bessell.txt'
    readcol, group, f='a', file, /silent
    for i=0, n_elements(file)-1 do begin
       readcol, fpath+file[i], lband, fband, /silent
       oplot, lband, fband, color=50, thick=cthick, linestyle=2
       xyouts, 8800, 0.85, 'Bessell (1990)',  align=0.5, charsize=pings.csize, color=50
    endfor
    ; CTIO
    group = fpath+'ALL_CTIO.txt'
    readcol, group, f='a', file, /silent
    for i=0, n_elements(file)-1 do begin
       readcol, fpath+file[i], lband, fband, /silent
       oplot, lband, fband, color=210, thick=cthick
       xyouts, 8800, 0.80, 'CTIO-SINGS',  align=0.5, charsize=pings.csize, color=210
    endfor
    ; KPNO
    group = fpath+'ALL_KPNO.txt'
    readcol, group, f='a', file, /silent
    for i=0, n_elements(file)-1 do begin
       readcol, fpath+file[i], lband, fband, /silent
       oplot, lband, fband, color=244, thick=cthick
       xyouts, 8800, 0.75, 'KPNO-Harris',  align=0.5, charsize=pings.csize, color=244
    endfor
   ; SDSS
    group = fpath+'ALL_SDSS.txt'
    readcol, group, f='a', file, /silent
    for i=0, n_elements(file)-1 do begin
       readcol, fpath+file[i], lband, fband, /silent
       oplot, lband, fband, color=5, thick=cthick
       xyouts, 8800, 0.70, 'SDSS-III',  align=0.5, charsize=pings.csize, color=5
    endfor
    if ps eq 1 then pingsoft_psoff

end


pro pingsoft_filters, filter, sband, lband, fband, cwl, USER=user, PLOT=plot

if keyword_set(plot) then begin
   plot_filters, PS=0
   return
endif

if n_params() lt 1 then begin
    print, ''
    print, '  PINGSoft FILTERS'
    print, ''
    print, '    1: Halpha KPNO-NOAO   | 6547 AA  80-FWHM'
    print, '    2: Halpha CTIO-SINGS  | 6586 AA  20-FWHM'
    print, ''
    print, '    3: B-Johnson (1965)   | 4400 AA'
    print, '    4: V-Johnson          | 5500 AA'
    print, '    5: R-Johnson          | 7000 AA'
    print, ''
    print, '    6: u-SDSS-III         | 3551 AA'
    print, '    7: g-SDSS-III         | 4686 AA'
    print, '    8: r-SDSS-III         | 6166 AA'
    print, '    9: i-SDSS-III         | 7480 AA'
    print, '   10: z-SDSS-III         | 8932 AA'
    print, ''
    print, '   11: B-Bessell (1990)   | 4520 AA'
    print, '   12: V-Bessell          | 5524 AA'
    print, '   13: R-Bessell          | 6535 AA'
    print, ''
    print, '   14: B-CTIO-SINGS       | 4350 AA'
    print, '   15: V-CTIO-SINGS       | 5550 AA'
    print, '   16: R-CTIO-SINGS       | 6500 AA'
    print, ''
    print, '   17: B-KPNO-Harris      | 4357 AA'
    print, '   18: V-KPNO-Harris      | 5375 AA'
    print, '   19: R-KPNO-Harris      | 6425 AA'
    print, ''
    print, ''
    ;---------- Plot
    plot_filters, PS=0
    return

endif

if keyword_set(user) then goto, user_key


;------------ Reading filters

case filter of
   
   1: sband='Halpha_KPNO-NOAO.txt'
   2: sband='Halpha_CTIO-SINGS.txt'
   
   3: sband='B_Johnson-1965.txt'
   4: sband='V_Johnson-1965.txt'
   5: sband='R_Johnson-1965.txt'

   6: sband='u_SDSS-III.txt'
   7: sband='g_SDSS-III.txt'
   8: sband='r_SDSS-III.txt'
   9: sband='i_SDSS-III.txt'
  10: sband='z_SDSS-III.txt'

  11: sband='B_Bessell-1990.txt'
  12: sband='V_Bessell-1990.txt'
  13: sband='R_Bessell-1990.txt'

  14: sband='B_CTIO-SINGS.txt'
  15: sband='V_CTIO-SINGS.txt'
  16: sband='R_CTIO-SINGS.txt'

  17: sband='B_KPNO-Harris.txt'
  18: sband='V_KPNO-Harris.txt'
  19: sband='R_KPNO-Harris.txt'

endcase


;------------ Reading filters

   path = expand_path('${PINGSOFT_PATH}')+'/filters/'
sfilter = path+sband

readcol, sfilter, lband, fband, /silent


;------------ Central wavelength
readcol, sfilter, f='a,a,f', v1,v2,v3, /silent
mm = where(v2 eq 'CWL:')
cwl = v3[mm]

sband = strsplit(sband, '.txt', /regex, /extract)

goto, fin


;------------------------ User filter

user_key:
readcol, user, lband, fband, /silent ; lambda, transmission
readcol, user, f='a,a,f', v1,v2,v3, /silent ; cwl
mm = where(v2 eq 'CWL:')
cwl = v3[mm]
sband = 'USER'

fin:
end
