


pro plot_segments, infile, seg_map, SEGMENT=segment, PS=ps, PLOT=plot, PREFIX=prefix, $
                  FILTER=filter, BAND=band, CT=ct, FONT=font,  $
                  CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                  SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                  DRAW=draw, NUMBER=number, x_arcsec=x_arcsec, y_arcsec=y_arcsec, _EXTRA=extra


; Prefix
tmp0 = infile
tmp1 = strsplit(tmp0,'_mask.cube.fits',/extract,/regex)
tmp2 = strsplit(tmp1,'.fits',/extract,/regex)
object = tmp2
if not keyword_set(prefix) then prefix=object[0]

ps_file = prefix+'_mask.FoV.eps'


; Segment map type
seg_type = size(seg_map,/tname)
if seg_type eq 'STRING' then smap = readfits(seg_map,/silent) else smap = seg_map

nreg = fix(max(smap))


;--------------------------  PINGSoft core

if keyword_set(ps) then tmp_ps = 1 else tmp_ps = 0
this_ps = 0


pingsoft_core, infile, pings, core, ps_file, $
               PT=pt, EXTENSION=extension, PREFIX=prefix, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=0, FONT=font, PLOT=0, $
               CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
               LARGE=large, NOEXTRACT=noextract, _EXTRA=extra


; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor

if keyword_set(segment) then scale = smap else scale = fmap
ps = tmp_ps


;-------------------  Vector construction

array = intarr(xx*yy)
sval = intarr(xx*yy)

nn = 0l
for j=0l, yy-1 do begin
   for i=0l, xx-1 do begin
      array[nn]=scale[i,j]
      nn = nn+1
   endfor
endfor

nn = 0l
for j=0l, yy-1 do begin
   for i=0l, xx-1 do begin
      sval[nn]=smap[i,j]
      nn = nn+1
   endfor
endfor    


;-------------------  Grepping values for borders

border1 = fltarr(xx,yy)
border2 = fltarr(xx,yy)
border3 = fltarr(xx,yy)
border4 = fltarr(xx,yy)

for k=1, nreg do begin

   for i=0, xx-1 do begin
      index = where(smap[i,*] eq k)
      if min(index) ge 0 then begin
         border1[i,min(index)] = 1
         border2[i,max(index)] = 1
      endif
   endfor
   
   for j=0, yy-1 do begin
      index = where(smap[*,j] eq k)
      if min(index) ge 0 then begin
         border3[min(index),j] = 1
         border4[max(index),j] = 1
      endif
   endfor

endfor

;top
xup = [-1,1,-1]*radius
yup = [ 1,1, 1]*radius
;bottom
xdown = [-1, 1,-1]*radius
ydown = [-1,-1,-1]*radius
;left
xleft = [ 1,1, 1]*radius
yleft = [-1,1,-1]*radius
;right
xright = [-1,-1,-1]*radius
yright = [-1, 1,-1]*radius



;--------------------------  Fibers in mosaic

!p.multi=[0,1,1]

loadct, 1, /silent


if keyword_set(ps) then begin
   
   if not keyword_set(font) then psfont=pings.font else psfont=font
   
   pingsoft_pson, ps_file, pings.x_ps, pings.y_ps, font=psfont, axis_thick=pings.axis, encapsulate=1
   
   plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
         xs=1, ys=1, charsize=pings.csize_ps, position=pings.pos0, $
         xtitle='!9D!X!e !nRA (arcsec)', ytitle='!9D!X!e !nDec (arcsec)', $
         title=title_str, _extra=extra
   goto, ps_on
   
endif


;-------------------  Windows position and dimensions

set_font
window, 0, xs=xwin0_size, ys=ywin0_size
plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, position=pings.pos0, charsize=pings.csize, $
      xtitle=pings.xtitle0, ytitle=pings.ytitle0, $
      title=pings.title0, /noerase, _extra=extra

ps_on:

;--------- Color tables 

if not keyword_set(ct) then ct = 1

spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'

ctables = [4,5,25,27,33,39,41,42]
ntest = where(ctables eq ct)
if ntest[0] ge 0 then full_ct = 1 else full_ct = 0


;-------------------  Plot regions

if not keyword_set(segment) then begin

   loadct, ct, file=ctfile, /silent

   for i=0l, n_elements(x)-1 do begin

      xcoor =  x[i]
      ycoor =  y[i]
      x_fib = xarr + xcoor
      y_fib = yarr + ycoor
      polyfill, x_fib, y_fib, color=array[i]

   endfor
   
   loadct, 1, /silent

endif



;-------------------  Plot segmentation map

if keyword_set(segment) then begin

   loadct, 38, /silent

   for k=1, nreg do begin

      h=where(smap eq k)

      for i=0l,n_elements(h)-1 do begin
         xp = xarr+x[h[i]]
         yp = yarr+y[h[i]]
         polyfill, xp, yp, color=250*((k+1)/18.)
      endfor
      ;if keyword_set(ps) then polyfill, xp, yp, color=250*((k+1)/18.)
   endfor

endif



;-------------------  Plot borders

if keyword_set(segment) then begin
   loadct, 1, /silent
   color = 0
endif else begin
   ctables = [4,5,6,15,33,42,44]
   ntest = where(ctables eq ct)
   if ntest[0] ge 0 then color = 255 else color = 250 
   loadct, 39, /silent
  ; color = 250
endelse

if keyword_set(ps) then thick = 5 else thick = 2

index=where(border1 eq 1)
for i=0l,n_elements(index)-1 do begin
   xp = xdown + x[index[i]]
   yp = ydown + y[index[i]]
   oplot, xp, yp, color=color, thick=thick
endfor

index=where(border2 eq 1)
for i=0l,n_elements(index)-1 do begin
   xp = xup + x[index[i]]
   yp = yup + y[index[i]]
   oplot, xp, yp, color=color, thick=thick
endfor

index=where(border3 eq 1)
for i=0l,n_elements(index)-1 do begin
   xp = xleft + x[index[i]]
   yp = yleft + y[index[i]]
   oplot, xp, yp, color=color, thick=thick
endfor

index=where(border4 eq 1)
for i=0l,n_elements(index)-1 do begin
   xp = xright + x[index[i]]
   yp = yright + y[index[i]]
   oplot, xp, yp, color=color, thick=thick
endfor


; Plot region number

if keyword_set(segment) then begin
   loadct, 1, /silent
   color = 0
endif else begin
   loadct, 38, /silent
   color = 220
endelse


if keyword_set(number) then begin
   if keyword_set(ps) then csize = 0.9 else csize = 0.5
   xreg = x_arcsec
   yreg = y_arcsec
   for k=0, nreg-1 do xyouts, xreg[k], yreg[k], sctrim(k+1), charthick=1.3, color=color, align=0.4, charsize=csize
endif

if keyword_set(ps) then begin
   pingsoft_psoff
   if not keyword_set(silent) then begin
      print, ''
      print, ' Postscript file written: '+ps_file
      print, ''
   endif
endif

end
