
pro show_hdr, infile, hdr, ASCII=ascii


;+
; NAME:
;      SHOW_HDR
;
; PURPOSE:
;
;      Shows on screen the header of a FITS file, which can be written to an ASCII file.
;
;
; CALLING SEQUENCE:
;       
;       show_hdr, 'file.FITS', [, hdr, ASCII='ascii']
;
; INPUTS:
;       
;       'file.FITS':  String with the name of the FITS file.
;
; OUTPUTS:
;
;               hdr:  IDL variable containing the header.
;
; OPTIONAL KEYWORDS:

;             ASCII:  String with the name of an ASCII file to write the header.
;
;
; EXAMPLES:
;
;       show_hdr, 'object.fits', header, ASCII='object_hdr.txt'
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings/
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "       show_hdr, 'file.FITS' [, hdr, ASCII='ascii']"
    print, ''
    print, ''
    print, "    'file.FITS':  String with the name of the FITS file."
    print, ''
    print, '            hdr:  IDL variable containing the header.'
    print, ''
    print, '          ASCII:  String with the name of an ASCII file to write the header.'
    print, ''
    return
endif


;--------------------------  Initial tests


infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file: '+infile+' does not exist!'
    print, ''
    return
endif


data = readfits(infile,hdr,/silent)

print, ''
print, 'FITS header of file:  ', infile
print, '-------------------------------------------------------------------------------'
print, hdr
print, ''


if keyword_set(ascii) then begin
    openw,lun,ascii, /get_lun
    printf,lun, 'FITS header of file:  ', infile
    printf,lun, '-------------------------------------------------------------------------------'
    for i=0,n_elements(hdr)-1 do printf,lun,hdr[i]
    close,lun
    print, ''
    print, 'FITS header written in: ', ascii
    print, ''
endif


end

