
function s2n_rat_func, clam, fluxc, fluxe, fcont1, fcont2, Pnoise, SQRTN=sqrtn, PROXY=proxy

  ; Baseline
  nn = n_elements(fluxe)
  nc1 = n_elements(fcont1)
  nc2 = n_elements(fcont2)
  nv = min([nn,nc1,nc2])
  fluxe  = fluxe[0:nv-1]
  fcont1 = fcont1[0:nv-1]
  fcont2 = fcont2[0:nv-1]

  ; Mean continuum associated to eline
  mean_cont = fltarr(nv)
  for k=0, nv-1 do mean_cont[k] = mean([fcont1[k],fcont2[k]])
  nok = where(mean_cont eq mean_cont)

  ; Signal eline
  fdiff = fluxe[nok] - mean_cont[nok]
  signal_e = total(fdiff) / sqrt(nv)

  ; Noise factors
  sigma1 = stddev(fcont1[nok])
  sigma2 = stddev(fcont2[nok])

  ; Poisson noise
  Pn = Pnoise * sqrt(fdiff)
  pp = where(Pn ne Pn)
  if n_elements(pp) gt 1 then Pn[pp] = 0.

  ; Noise eline
  sigma_e = mean([sigma1,sigma2])
  sigma_P = stddev(Pn)
  if sigma_P ne sigma_P then sigma_P = mean(Pnoise) * mean(fdiff)
  noise_e = sqrt( sigma_e^2 + sigma_P^2 )

  ; S/N eline
  s2n_e = signal_e / noise_e
  if s2n_e ne s2n_e then begin
        s2n_e = 0
     signal_e = 0
      noise_e = 0
   endif
  if s2n_e lt 0 then begin
        s2n_e = 0
     signal_e = 0
      noise_e = 0
   endif

  ; Detrended S/N ratio
  ab = linfit(clam,fluxc)
  y = ab[0] + ab[1]*clam

  signal_c = mean(fluxc)
  noise_c = stddev(fluxc-y)

  if keyword_set(sqrtn) then noise_c = sqrt(stddev(fluxc-y))

  if keyword_set(proxy) then begin
     signal_c = total(fluxc) 
     noise_c = sqrt(signal_c)
  endif

  s2n_c = signal_c / noise_c

  if s2n_c ne s2n_c then begin
        s2n_c = 0
     signal_c = 0
      noise_c = 0
   endif
  if s2n_c lt 0 then begin
        s2n_c = 0
     signal_c = 0
      noise_c = 0
   endif

  s2n = [ s2n_c, s2n_e, signal_c, signal_e, noise_c, noise_e ]

return, s2n
end

