

pro plot_gui, s, skip=skip, load=load, _extra=extra

  common widget, main, hdr_wcs, coord, wlmin, wlmax, slide, wband, fits, wpt, obj, status

  ; No graphics needed, just spectra explorer
  if keyword_set(skip) then goto, skip_core

  ; New FITS loaded
  if keyword_set(load) then begin
          
     widget_control, status, set_value='Loading new FITS...'

     infile = load.infile
     ptable = load.ptable
     pingsoft_param, pings, ''
     pingsoft_core, infile, pings, core, PT=ptable, LOAD=load, PLOT=1

     s = core
     str1 = 'Object:  '+s.object[0]
     if s.califa eq 1 then str1 = 'CALIFA object: '+s.object[0]
     str_lmin = sctrim(s.lmin,format='(f7.1)')
     str_lmax = sctrim(s.lmax,format='(f7.1)')
   
     widget_control, obj, set_value=str1
     widget_control, hdr_wcs, set_value=s.hdr_coor
     widget_control, wlmin, set_value=str_lmin
     widget_control, wlmax, set_value=str_lmax
     widget_control, slide, set_slider_min=s.lmin
     widget_control, slide, set_slider_max=s.lmax

     widget_control, main, set_uvalue=s

   goto, skip_core

endif

; New graphics needed
pingsoft_core, infile, pings, core,  WIDGET=s, _EXTRA=extra


; Reading output variables to main structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute('s.'+dumb+' = core.'+dumb)
endfor
widget_control, main, set_uvalue=s


skip_core:
; Variables from structure
tag = get_tags(s)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = s.'+dumb)
endfor
widget_param, wparam
pingsoft_param, pings, ''


; lmin/lmax
indice = where(lam ge lmin and lam le lmax)
lambda = lam[indice]
xmin = min(lambda) & xmax = max(lambda)


; Postscript
if keyword_set(ps) then begin
   pingsoft_psoff
   pingsoft_display, pings, /silent
   goto, fin_gui
endif

widget_control, status, set_value= 'Explorer ON, RIGHT-click to access options'

!MOUSE.button=0
write = 0
extract = long(0)

; Special colors
ctables = [4,5,6,15,33,42,44]
ntest = where(ctables eq ct)
if ntest[0] ge 0 then thisCol = 255 else thisCol = 250 

; Status
if region eq 1 then widget_control, status, set_value= 'Select spaxels to extract with LEFT-click'


while (!MOUSE.button ne 4) do begin

wset, windex2
    cursor, xm, ym, /change

    x_min = xm-radius
    x_max = xm+radius
    y_min = ym-radius
    y_max = ym+radius

    index = where(x ge x_min and x le x_max and y ge y_min and y le y_max)

    if n_elements(index) eq 1 then begin
        test = index[0]
        if test lt 0 then goto,next
    endif

    id_ok  = id[index]
    x_test = x[index]
    y_test = y[index]
    d = sqrt((x_test-xm)^2. + (y_test-ym)^2.)
    ind = where(d eq min(d))
    fibre = fix(id_ok[ind]-1,type=3)
    fibre = fibre[0]

    xcoor =  x[fibre]
    ycoor =  y[fibre]

    flux = data[*,fibre]
    if keyword_set(califa) and bpix[0] ne -1 then fpix = bpix[*,fibre]

    if not keyword_set(min_flux) then a_flux = min(flux[indice]) else a_flux = min_flux

    if not keyword_set(max_flux) then y_flux = max(flux[indice])*1.2 else y_flux = max_flux

    wset, windex1

    sfibre = sctrim(fibre)
    if type eq 2 then vtitle='!5Spaxel ID: '+sfibre+'!X'

    if type eq 3 then begin
       ncol = xx
       col = (fibre MOD ncol)+1
       row = (fibre / ncol)+1
       scol = sctrim(col)
       srow = sctrim(row)
       vtitle='!5Spaxel: ['+scol+','+srow+']    ID: '+sfibre+'!X'
    endif

    plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[a_flux,y_flux], xs=1, ys=1, $
          xthick=wparam.wthick, ythick=wparam.wthick, pos=wparam.pos2, charsize=wparam.csize, $
          xtitle=wparam.xtitle2, ytitle=wparam.ytitle2

    ; Filter
    loadct, 0, /silent
    oplot, lam, res*y_flux, linestyle=pings.bline, color=pings.bcolor
    loadct, 1, /silent

    if keyword_set(vline) then begin
       if size(vline,/tname) eq 'STRING' then com=execute(vline)
       loadct, wparam.ctline, /silent
       if n_elements(vline) gt 1 then for k=0, n_elements(vline)-1 do $
          plots, [vline[k],vline[k]],[a_flux,y_flux], color=wparam.cline, thick=wparam.tline $
       else $
          plots, [vline,vline],[a_flux,y_flux], color=wparam.cline, thick=wparam.tline
       loadct, 1, /silent
    endif

    oplot, lambda, flux[indice], color=wparam.fcolor
    if keyword_set(califa) and bpix[0] ne -1 then oplot, lambda, flux[indice]*fpix[indice], color=wparam.bpcolor, min_value= 1e-6
    next:

    wset, windex2
    plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
          xs=1, ys=1, xthick=wparam.wthick, ythick=wparam.wthick, pos=wparam.pos0, charsize=wparam.csize, $
          xtitle=xtitle_w0, ytitle=ytitle_w0, /noerase


    ; Spectra extraction
    if region eq 1 then begin
       write = 1
       widget_control, status, set_value= 'Select spaxels to extract with LEFT-click'
    endif

    if n_elements(index) eq 1 then begin
       test = index[0]
       if test lt 0 then goto, out_range
    endif
    
    if wcs eq 0 then wcs_coord1 = ''
    if wcs eq 0 then wcs_coord2 = ''
    
    if wcs eq 1 then begin
       
            coord1 = offset2radec( ref_ra, ref_dec, xcoor, ycoor, /SEXAG)
            
            if coord1[3] lt 0 or coord1[4] lt 0 or coord1[5] lt 0 then sign = '-' else sign=' '
            str_0 = string(coord1[0],format='(i02)')
            str_1 = string(coord1[1],format='(i02)')
            str_2 = string(coord1[2],format='(f04.1)')
            str_3 = sign+string(abs(coord1[3]),format='(i02)')
            str_4 = string(abs(coord1[4]),format='(i02)')
            str_5 = string(abs(coord1[5]),format='(f04.1)')
            wcs_coord1 = str_0+'h '+str_1+'m '+str_2+'s  '+str_3+'d '+str_4+'m '+str_5+'s'
            
            coord2 = offset2radec( ref_ra, ref_dec, xcoor, ycoor )
            ra_deg = coord2[0]
            dec_deg = coord2[1]
            wcs_coord2 = sctrim(ra_deg)+'   '+sctrim(dec_deg)
            
         endif
    
    s_fibre = string(fibre,format='(i6)')
    s_xcoor = string(xcoor,format='(f7.1)')
    s_ycoor = string(ycoor,format='(f7.1)')
    
    if type eq 2 then $
       str_coor = '    '+s_fibre+'    '+s_xcoor+'  '+s_ycoor+'    '+wcs_coord1+'    '+wcs_coord2
    if type eq 3 then begin
           s_col = string(col,format='(i3)')
           s_row = string(row,format='(i3)')
           str_coor = '  ['+s_col+','+s_row+']   '+s_fibre+'    '+s_xcoor+'  '+s_ycoor+'    '+wcs_coord1+'    '+wcs_coord2
        endif

    if !MOUSE.button eq 1 then begin
       if write eq 0 then begin
          print, hdr_coor
          print, str_coor
          print, ''
       endif
       if write eq 1 then begin
          extract = [extract, fibre] 
          loadct, 39, /silent
          oplot, xarr+xcoor, yarr+ycoor, thick=2, color=thisCol
          polyfill, xarr+xcoor, yarr+ycoor, /line_fill, color=thisCol, orient=45, spacing=0.1
          loadct, 1, /silent
       endif
    endif

    out_range:
    
    widget_control, coord, set_val=str_coor

endwhile


if write eq 1 then begin

    extract = extract[1:*]

    ; ignoring repeated fibers
    new = long(0)
    for k=0l, n_elements(extract)-1 do begin
        test = extract[k]
        for n=k+1,  n_elements(extract)-1 do begin
            if test eq extract[n] then goto, incl
        endfor
        new = [new,test]
        incl:
    endfor

    new = fix(new[1:*],type=3)
    extract = new[sort(new)]
 
    integrated = fltarr(pixels)
    rss_data = fltarr(pixels,n_elements(extract))

    for i=0l, n_elements(extract)-1 do begin
        index_integ = fix(extract[i],type=3)
        integrated = integrated + data[*,index_integ]
        rss_data[*,i] = data[*,index_integ]
    endfor

    prefix = object

    ; Integrated flux ASCII
    out_integ = prefix+'_integ.txt'
    openw,3,out_integ
    for k=0l, n_elements(integrated)-1 do printf,3, k+1, lam[k], integrated[k]
    close, 3

    ; Integrated flux FITS
    out_fits = prefix+'_integ.fits'
    writefits, out_fits, integrated, hdr1

    ; RSS FITS file
    out_rss = prefix+'_rss.fits'
    writefits, out_rss, rss_data, hdr2

    ; New position table & indices
    out_pt = prefix+'_rss.pt.txt'
    out_index = prefix+'_index.txt'

    openw, 1, out_pt
    openw, 2, out_index

    if mtype eq 'C' then printf, 1, 'C  '+sctrim(xsize)+'  '+sctrim(xsize)+'  0'
    if mtype eq 'S' then printf, 1, 'S  '+sctrim(xsize)+'  '+sctrim(xsize)+'  0'
    
    for i=0l, n_elements(extract)-1 do begin
        nn = extract[i]
        printf, 1, sctrim(i+1), x[nn], y[nn], 1
        printf, 2, extract[i]
    endfor

    ; PS extracted spectrum
    out_ps =  prefix+'_integ.eps'
    pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
    loadct, 1, /silent
    plot, [0], /nodata, xr=[min(lam)-80,max(lam)+80], yr=[min(integrated), max(integrated)*1.2], $
          xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
          xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
    oplot, lam, integrated, color=100
    pingsoft_psoff
    
    wset, windex1
    plot, [0], /nodata, xr=[min(lam)-80,max(lam)+80], yr=[min(integrated), max(integrated)*1.2], $
          xs=1, ys=1, xthick=wparam.wthick, ythick=wparam.wthick, pos=wparam.pos2, charsize=wparam.csize, $
          xtitle='Wavelength', ytitle='Flux'
    loadct, 39, /silent
    oplot, lam, integrated, color=250

    print, ''
    print, ' PINGSoft'
    print, ' ==================='
    print, ''
    print, ' Files written '
    print, ''
    print, '      Extracted RSS:   ', out_rss
    print, '     Position table:   ', out_pt
    print, '   Integrated ASCII:   ', out_integ
    print, '               FITS:   ', out_fits
    print, '         Postscript:   ', out_ps
    print, '        IDL indices:   ', out_index
    print, ''

pingsoft_display, pings, /silent

endif
close, /all

widget_control, status, set_value= 'Explorer OFF, widget options active'

fin_gui:
end


pro view_ifs_event, event

common widget, main, hdr_wcs, coord, wlmin, wlmax, slide, wband, fits, wpt, obj, status


widget_control, event.top, get_uvalue=s

widget_control, event.id, get_uvalue=uvalue

case uvalue of


   ;---------------------------------- Pick FITS
   'pick1' : begin
      infile = dialog_pickfile( filter='*.fits,*.gz', title='Choose FITS file', get_path=path)
      tmp0 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
      tmp1 = strsplit(tmp0,'.rscube.fits.gz',/extract,/regex)
      tmp2 = strsplit(tmp1,'.rscube.fits',/extract,/regex)
      tmp3 = strsplit(tmp2,'.fits.gz',/extract,/regex)
      tmp4 = strsplit(tmp3,'.fits',/extract,/regex)
      test_pt = path+tmp4+'.pt.txt'

      sfile = strsplit(infile, path, /regex, /extract)
      s_pt = strsplit(test_pt, path, /regex, /extract)

      if strlen(infile) gt 0 then begin

         s.infile = infile
         s.path = path
         widget_control, fits, set_value=sfile
         if file_test(test_pt) eq 1 then begin
            s.pt = test_pt 
            widget_control, wpt, set_value=s_pt
         endif else begin
            widget_control, wpt, set_value=''
         endelse
         widget_control, event.top, set_uvalue=s

      endif else begin
         s.infile = ''
         s.pt = ''
         widget_control, event.top, set_uvalue=s
      endelse
   end


   ;---------------------------------- Pick PosTable
   'pick2' : begin
      file_pt = dialog_pickfile( filter='*.pt.txt', title='Choose Position Table file', get_path=path)
      s_pt = strsplit(file_pt,path,/extract,/regex)
      if strlen(file_pt) gt 0 then begin
         s.pt = file_pt
      endif else begin
         s.pt = ''
      endelse
      widget_control, wpt, set_value=s_pt
      widget_control, event.top, set_uvalue=s
   end

   ;---------------------------------- Load FITS
   'load' : begin
      if file_test(s.infile) eq 1 then begin
         load = { $
                infile:s.infile   ,$
                ptable:s.pt       ,$
                windex1:s.windex1 ,$
                windex2:s.windex2 }
         print, ''
         print, ' PINGSoft IFS viewer'
         print, ' ==================='
         print, ''
         print, ' Loading new FITS...'
         print, ''
         plot_gui, s, load=load
      endif
   end

   ;---------------------------------- Flux range
   'fmin' : begin
      s.min_flux = event.value
      widget_control, event.top, set_uvalue=s
   end
   'fmax' : begin
      s.max_flux = event.value
      widget_control, event.top, set_uvalue=s
   end
   'set1' : begin
      str_fmin = sctrim(s.min_flux)
      str_fmax = sctrim(s.max_flux)
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ==================='
      print, ''
      print, '  New flux range set '
      print, ''
      if s.min_flux ne 0 then $
      print, '      Min:  '+str_fmin
      if s.max_flux ne 0 then $
      print, '      Max:  '+str_fmax
      print, ''
      plot_gui, s, skip=1
   end
   'reset1' : begin
      s.min_flux = 0.
      s.max_flux = 0.
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ==================='
      print, ''
      print, '   Flux range RESET'
      print, ''
      widget_control, event.top, set_uvalue=s
      plot_gui, s, skip=1
   end

   ;---------------------------------- Wavelength range
   'lmin' : begin
      s.lmin = event.value
      widget_control, event.top, set_uvalue=s
   end
   'lmax' : begin
      s.lmax = event.value
      widget_control, event.top, set_uvalue=s
   end
   'set2' : begin
      str_lmin = sctrim(s.lmin,format='(f7.1)')
      str_lmax = sctrim(s.lmax,format='(f7.1)')
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ========================'
      print, ''
      print, ' New wavelength range set '
      print, ''
      print, '        Min:  '+str_lmin
      print, '        Max:  '+str_lmax
      print, ''
      widget_control, slide, set_slider_min=s.lmin
      widget_control, slide, set_slider_max=s.lmax
      plot_gui, s, skip=1
   end
   'reset2' : begin
      s.lmin = min(s.lam)
      s.lmax = max(s.lam)
      str_lmin = sctrim(s.lmin,format='(f7.1)')
      str_lmax = sctrim(s.lmax,format='(f7.1)')
      widget_control, wlmin, set_value=str_lmin
      widget_control, wlmax, set_value=str_lmax
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ========================'
      print, ''
      print, ' New wavelength range set '
      print, ''
      print, '        Min:   '+str_lmin
      print, '        Max:   '+str_lmax
      print, ''
      widget_control, event.top, set_uvalue=s
      widget_control, slide, set_slider_min=s.lmin
      widget_control, slide, set_slider_max=s.lmax
      plot_gui, s, skip=1
   end

   ;---------------------------------- Scalings
   'linear' : begin
      if event.select eq 1 then begin
         s.linear = 1 
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.linear = 0
         widget_control, event.top, set_uvalue=s
      endelse
   end
   'clip' : begin
      if event.select eq 1 then begin
         s.clip = 1 
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.clip = 0
         widget_control, event.top, set_uvalue=s
      endelse
   end
   'asinh' : begin
      if event.select eq 1 then begin
         s.asinh = 1 
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.asinh = 0
         widget_control, event.top, set_uvalue=s
      endelse
   end
   'gamma' : begin
      if event.select eq 1 then begin
         s.gamma = 1
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.gamma = 0
         widget_control, event.top, set_uvalue=s
      endelse
   end
   'sqrt' : begin
      if event.select eq 1 then begin
         s.sqrt = 1
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.sqrt = 0
         widget_control, event.top, set_uvalue=s
      endelse
   end
   'histogram' : begin
      if event.select eq 1 then begin
         s.histogram = 1 
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.histogram = 0
         widget_control, event.top, set_uvalue=s
      endelse
   end
   'gaussian' : begin
      if event.select eq 1 then begin
         s.gaussian = 1 
         widget_control, event.top, set_uvalue=s
         plot_gui, s
      endif else begin
         s.gaussian = 0
         widget_control, event.top, set_uvalue=s
      endelse 
   end

  ;---------------------------------- Color Table
   'ct' : begin
      case event.index of
         0:  s.ct = 1    ; Blue
         1:  s.ct = 3    ; Red
         2:  s.ct = 8    ; Green
         3:  s.ct = 99
         4:  s.ct = 44   ; PINGSoft
         5:  s.ct = 42   ; CALIFA
         6:  s.ct = 33   ; Rainbow
         7:  s.ct = 4    ; BGRY
         8:  s.ct = 6    ; Prism
         9:  s.ct = 15   ; Stern
        10:  s.ct = 5    ; Std-Gamma
      endcase
      widget_control, event.top, set_uvalue=s
      plot_gui, s
   end

   ;---------------------------------- Filter
   'filter' : begin
      case event.index of
         0:  s.filter = 1
         1:  s.filter = 2
         2:  s.filter = 3
         3:  s.filter = 4
         4:  s.filter = 5
         5:  s.filter = 6
         6:  s.filter = 7
         7:  s.filter = 8
         8:  s.filter = 9
         9:  s.filter = 11
        10:  s.filter = 12
        11:  s.filter = 13
        12:  s.filter = 17
        13:  s.filter = 18
        14:  s.filter = 19
     endcase
      pingsoft_filters, s.filter, sband, lband, fband, cwl
      s.band = 0
      if cwl lt min(s.lam) then cwl = min(s.lam)
      if cwl gt max(s.lam) then cwl = max(s.lam)
      widget_control, slide, set_value=cwl
      widget_control, wband, set_value=cwl
      widget_control, event.top, set_uvalue=s
      plot_gui, s
   end
;caro
   ;---------------------------------- Mark wavelength (vline)
   'vline' : begin
      test = event.value[0]
      ;removing a possible ',' at the end of the string
      test1 = strlen(test)
      test2 = strpos(test,',',/reverse_search)
      if test2 gt 0 then begin
         if test1 eq test2+1 then $
            value = strmid(test,0,test2) $
         else $
            value = test
      endif else begin
         value = test
      endelse
      s.vline = 'vline=['+value+']'
      ;removing possible leading spaces
      if sctrim(value) eq '' then s.vline=''
      widget_control, event.top, set_uvalue=s
   end
   'set3' : begin
      str_vline = strsplit(s.vline,'vline=',/extract,/regex)
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ====================='
      print, ''
      print, ' Marking wavelength(s):  '+str_vline
      print, ''
      plot_gui, s, skip=1
   end

   ;--------------------------------- Draw
   'draw' : begin
      if event.select eq 1 then s.draw = 1 else s.draw = 0
      widget_control, event.top, set_uvalue=s
      plot_gui, s
   end

   ;--------------------------------- Invert Color Map
   'inverse' : begin
      if event.select eq 1 then s.inverse = 1 else s.inverse = 0
      widget_control, event.top, set_uvalue=s
      plot_gui, s
   end

   ;---------------------------------- Central wavelength
   'slide' : begin
      s.band = event.value
      str_band = sctrim(s.band)
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ============================='
      print, ''
      print, ' Filter shifted to wavelength:  '+str_band
      print, ''
      widget_control, event.top, set_uvalue=s
      widget_control, wband, set_value=s.band
      plot_gui, s
   end
   'wband' : begin
      s.band = event.value
      str_band = sctrim(s.band)
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ============================'
      print, ''
      print, ' Filter shifted to wavelength:  '+str_band
      print, ''
      widget_control, event.top, set_uvalue=s
      widget_control, slide, set_value=s.band
      plot_gui, s
   end

   ;---------------------------------- Extract region/slice
   'region' : begin
      s.region = 1
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ==================='
      print, ''
      print, ' Storing all spaxel positions with a left-click'
      print, ''
      widget_control, status, set_value= 'Select spaxels to extract with LEFT-click'
      widget_control, event.top, set_uvalue=s
      plot_gui, s, skip=1
      s.region = 0
      widget_control, event.top, set_uvalue=s
      widget_control, status, set_value= 'Region extractd, output files shown in terminal'
   end
   'slice' : begin
      str_slice = s.object+'_slice'
      out_fits = str_slice+'.fits'
      extract_filter,  s.infile, s.filter, s.band, prefix=str_slice, /widget
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ============================'
      print, ''
      print, ' Extracting FITS image slice:  '+out_fits
      print, '                      Filter:  '+s.sband
      print, '                  Wavelength:  '+sctrim(s.band)
      print, ''
      widget_control, status, set_value='FITS image slice extracted:  '+out_fits[0]
   end

   ;---------------------------------- Postscript
   'ps' : begin
      s.ps = 1
      ps_file = s.object+'_FoV.eps'
      widget_control, event.top, set_uvalue=s
      plot_gui, s, _EXTRA={title:''}
      print, ''
      print, ' PINGSoft IFS viewer'
      print, ' ============================='
      print, ''
      print, ' FoV saved in Postscript file:  '+ps_file
      print, ''
      spawn, 'mv idl.ps '+ps_file
      s.ps = 0
      widget_control, event.top, set_uvalue=s
      widget_control, status, set_value='FoV saved in Postscript file:  '+ps_file[0]
   end

   ;---------------------------------- QUIT
   'quit' : begin
      if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'
      widget_control, event.top, /destroy
      goto, fin
   end


end

widget_control, event.id, set_uvalue=uvalue, /realize

fin:
end



pro view_IFS, infile, PT=pt, HELP=help


compile_opt idl2, hidden
on_error, 2

common widget, main, hdr_wcs, coord, wlmin, wlmax, slide, wband, fits, wpt, obj, status

;-------------------------- Help

if keyword_set(help) then begin
   print, ''
   print, ' PINGSoft IFS viewer'
   print, ' -------------------'
   print, ''
   print, ' This routine provides a spatial and spectral interactive'
   print, ' visualisation widget for 3D cubes and RSS IFS files.'
   print, ''
   print, ' If the command is entered with no parameters, it prompts'
   print, ' for an input FITS file using a dialog window. Otherwise, '
   print, ' the input file can be passed directly to the command as the'
   print, ' first parameter:'
   print, ''
   print, " view_ifs, 'OBJECT.fits'"
   print, ''
   print, ''
   print, ' The widget will be displayed automatically if the input file is a 3D FITS'
   print, ' cube. If the input FITS is a RSS file, the program will look in the same'
   print, ' directory for a position table named OBJECT.pt.txt and will launch the widget'
   print, ' if the file exists. If this is not the case, the program will exit with an'
   print, ' error. The user can define the name of the corresponding position table using'
   print, " the PT='PosTable.txt' parameter:"
   print, ''
   print, " view_ifs, 'OBJECT.fits', PT='PosTable.txt'"
   print, ''
   print, ''
   print, ' USAGE: When the widget is launched, the user can explore'
   print, ' spatially and spectrally the IFS data but the widget options will be'
   print, ' "inactive". To have access to the widget options, the user'
   print, ' needs to RIGHT-click with the mouse over the FoV panel. When the widget'
   print, ' options are changed the spectral explorer will become active again and the'
   print, ' widget options will be unavailable. To active/deactivate the widget options the'
   print, ' user only needs to RIGHT-click over the FoV to switch between the explorer'
   print, ' ON/OFF options. The status bar above the FoV panel (below the object name)'
   print, ' will indicate whether the explorer is active or not.'
   print, ''
   print, ''
   print, ' Have a nice visualisation!'
   print, ''
   return
endif


;--------------------------  Pick file if null entry

if n_params() lt 1 then $
   infile = dialog_pickfile( filter='*.fits,*.gz', title='Choose FITS file', get_path=path )


;--------------------------  PINGSoft core

pingsoft_core, infile, pings, core, PT=pt

; widget structure
s = core

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor

widget_param, wparam

spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'

read_wcs = offset2radec(ref_ra,ref_dec,0,0,/SEXAG) ; useless, just compiling functions
dumb = sctrim('pingsoft')


;-----------------------------------------------  User variables and structure

filters = ['Halpha-KPNO: 6547-80A', 'Halpha-CTIO: 6586-20A', $
          'B-Johnson (1965)', 'V-Johnson', 'R-Johnson', $
          'u-SDSS-III', 'g-SDSS-III', 'r-SDSS-III', 'i-SDSS-III', $
          'B-Bessell (1990)', 'V-Bessell', 'R-Bessell', $
          'B-KPNO-Harris', 'V-KPNO-Harris', 'R-KPNO-Harris' ]


ctmap = ['Blue-scaling', 'Red-scaling', 'Green-scaling', 'Grayscale',  $
         'PINGSoft-special', 'CALIFA-special', 'Rainbow', 'BGRY', 'Prism', 'Stern', 'Std-Gamma' ]



;-----------------------------------------------  GUI Definition elements

if wparam.xdim lt 1280 or wparam.ydim lt 1024 then begin
   print, ''
   print, ''
   print, ' WARNING: Screen resolution too small for widget (min: 1280x1024)'
   print, ''
   print, '   xsize: '+sctrim(fix(wparam.xdim))
   print, '   ysize: '+sctrim(fix(wparam.ydim))
   print, ''
endif

xsize = wparam.xsize
ysize = wparam.ysize
xoff = wparam.xoff
yoff = wparam.yoff

xlogo = 205
ylogo = xlogo*1.05

yrow1 = 240
yrow2 = 50
yrow3 = 400
yrow4 = 45

if wparam.ydim lt 1024 then begin
   yrow2 = 45
   yrow3 = 300
endif

xlam = fix(xsize*0.55)
xFoV = fix(xsize*0.4)


main  = widget_base(title='PINGSoft:  IFS viewer',  $
                    xsize=xsize, ysize=ysize, xoff=xoff, yoff=yoff, units=0, /column, /frame)


; Logo
logo = read_image( '$PINGSOFT_PATH/PINGSoft.jpg' )
if califa eq 1 then $
   logo = read_image( '$PINGSOFT_PATH/CALIFA.jpg' ) 


;---------------------- Main bases

 base = widget_base(main, /row)
row_1 = widget_base(base, /column, xsize=xsize, ysize=yrow1 )

 base = widget_base(main, /row)
row_2 = widget_base(base, /column )

 base = widget_base(main, /row)
row_3 = widget_base(base, /column )

 base = widget_base(main, /row)
row_4 = widget_base(base, /column )


;-------------------------------------------- Row 1

 base = widget_base(row_1, /row, /align_top )

; Logo
base0 = widget_base(base, /column, ysize=ylogo )
dumb = widget_base(base0, /column, /frame )
win0 = widget_draw(dumb, xsize=xlogo, ysize=ylogo )


;----------------  Pick-up files

; Space
dumb = widget_base(base, /column, xsize=0)

base0 = widget_base(base, /column, /align_top )

 dumb = widget_base(base0, /row,  /frame )
pick1 = widget_button(dumb, ysize=35, value='Browse:', uvalue='pick1' )
 fits = cw_field(dumb, xsize=50, title='', value=tmp0, uvalue='fits', /all_events)
 dumb = widget_label(dumb, value=' FITS file ', /align_left)
 
if type eq 2 then ptvalue = strmid(pos_table,strpos(pos_table,'/',/reverse_search)+1)
if type eq 3 then ptvalue = ''

 dumb = widget_base(base0, /row, /frame )
pick2 = widget_button(dumb, ysize=35, value='Browse:', uvalue='pick2')
  wpt = cw_field(dumb, xsize=50, title='', value=ptvalue, uvalue='pt', /all_events)
 dumb = widget_label(dumb, value=' Position Table ', /align_left)


;----------------  Lambda/Flux MIN/MAX

base1 = widget_base(base0, /row )

; Hspace
dumb = widget_base(base1, xsize=2, /column)

; Flux
base2 = widget_base(base1, /column, /base_align_left, /frame )
dumb = widget_label(base2, value='Flux range', /align_left)
base3 = widget_base(base2, /row)
wfmin = cw_field(base3, xsize=10, title=' Min:',  uvalue='fmin', /all_events, /float)
 set1 = widget_button(base3,  value='Set', uvalue='set1', /align_center )
base3 = widget_base(base2, /row)
wfmax = cw_field(base3, xsize=10, title=' Max:',  uvalue='fmax', /all_events, /float)
reset1 = widget_button(base3,  value='Reset', uvalue='reset1', /align_center )

;space
dumb = widget_base(base1, xsize=2, /column)

; Wavelength
base2 = widget_base(base1, /column, /base_align_left, /frame )
dumb = widget_label(base2, value='Wavelength range', /align_left)
base3 = widget_base(base2, /row)
str_lmin = sctrim(xmin,format='(f5.0)')
str_lmax = sctrim(xmax,format='(f5.0)')
wlmin = cw_field(base3, xsize=10, title=' Min:', value=str_lmin, uvalue='lmin',  /all_events, /float)
 set2 = widget_button(base3,  value='Set', uvalue='set2', /align_center )
base3 = widget_base(base2, /row)
wlmax = cw_field(base3, xsize=10, title=' Max:', value=str_lmax, uvalue='lmax',  /all_events, /float)
reset2 = widget_button(base3,  value='Reset', uvalue='reset2', /align_center )


;----------------  Load/Quit Buttons

;space
dumb = widget_base(base1, xsize=15, /column)

base2 = widget_base(base1, /column )

base3 =  widget_base(base2, /column, /frame )
  ok = widget_button(base3, xsize=100, ysize=40,  value='Load FITS', uvalue='load', /align_center )

;space
dumb = widget_base(base2, ysize=2, /column)

base3 =  widget_base(base2, /column, /frame )
 quit = widget_button(base3, xsize=100, ysize=40,  value='Quit', uvalue='quit', /align_center )


;----------------  Scaling

; Space
dumb = widget_base(base, /column, xsize=1)

base0 = widget_base(base, /column,  /align_top, /frame)
 dumb = widget_label(base0, value='Scaling', /align_left)
base1 = widget_base(base0, /column, /exclusive)

   linear = widget_button(base1, value='Linear', uvalue='linear')
   clip = widget_button(base1, value='2% clipping', uvalue='clip')
    asinh = widget_button(base1, value='Asinh', uvalue='asinh')
    gamma = widget_button(base1, value='Power-Law', uvalue='gamma')
     sqrt = widget_button(base1, value='Square Root', uvalue='sqrt')
histogram = widget_button(base1, value='Hist Equal', uvalue='histogram')
 gaussian = widget_button(base1, value='Gaussian', uvalue='gaussian')

widget_control, linear, /set_button

; Space
dumb = widget_base(base0, /column, ysize=1)



;----------------  Display

; Space
dumb = widget_base(base, /column, xsize=1)

base0 = widget_base(base, /column,  /align_top, /frame)
 dumb = widget_label(base0, value='Display', /align_left)

base1 = widget_base(base0, /column )

base2 = widget_base(base1, /row )
 dumb = widget_label(base2, value=' Color Map:', /align_left)
 dumb = widget_droplist(base2, value=ctmap, uvalue='ct')

base2 = widget_base(base1, /row )
 dumb = widget_label(base2, value=' Filter:', /align_left)
 dumb = widget_droplist(base2, value=filters, uvalue='filter')

base2 = widget_base(base1, /row )
vline = cw_field(base2, title='Mark wavelength:', xsize=12, uvalue='vline', /all_events)
 set3 = widget_button(base2,  value='Set', uvalue='set3', /align_center )

base2 = widget_base(base1, /row, /nonexclusive )
 dumb = widget_button(base2, value='Draw spaxels contours', uvalue='draw')

base2 = widget_base(base1, /row, /nonexclusive )
 dumb = widget_button(base2, value='Invert Color Map', uvalue='inverse')


;-------------------------------------------- Row 2

base = widget_base(row_2, /row, ysize=yrow2, /base_align_center )


str_coor = '                                                                                                   '
base0 = widget_base(base, /column, xsize=xlam )
base1 = widget_base(base0, /column, /frame )
hdr_wcs = widget_label(base1, value=hdr_coor, xsize=640, /align_left)
  coord = widget_label(base1, value=str_coor, /align_left )

;Space
 dumb = widget_base(base, /row, xsize=15 )

str1 = 'Object:  '+object[0]
if califa eq 1 then str1 = 'CALIFA object: '+object[0]

str2 = 'Explorer ON, right-click to access options'
base0 = widget_base(base, /row, xsize=xFov  )
base1 = widget_base(base0, /column, xsize=xFov*0.95, /frame, /align_center )
   obj = widget_label(base1, value=str1, xsize=xFov*0.9 )
status = widget_label(base1, value=str2, xsize=xFov*0.9 )



;-------------------------------------------- Row 3

 base = widget_base(row_3, /row, /base_align_center )

col_1 = widget_base(base, /column, /frame )
 win1 = widget_draw(col_1, xsize=xlam, ysize=yrow3)

 dumb = widget_base(base, /row, xsize=1 )

col_2 = widget_base(base, /column, /frame, /align_center)
 win2 = widget_draw(col_2, xsize=xFoV, ysize=yrow3)


;-------------------------------------------- Row 4

 base = widget_base(row_4, /row, /base_align_center, ysize=yrow4 )

col_1 = widget_base(base, /row, xsize=xlam, /frame )

wband = cw_field(col_1, title='Shift Filter: ', xsize=10, value=s.band, uvalue='wband', /return_events)
slide = widget_slider(col_1, min=s.lmin, max=s.lmax, value=s.band,  uvalue='slide',  xsize=xlam*0.7 )

;Space
 dumb = widget_base(base, /row, xsize=1 )

col_2 = widget_base(base, /row, xsize=xFoV )

base1 = widget_base(col_2, /column, xsize=140, /align_center )
base2 = widget_base(base1, /column, /frame, /align_center)
region = widget_button(base2, xsize=120, ysize=35,  value='Extract region', uvalue='region', /align_center )

dumb = widget_base(col_2, /column, xsize=5)

base1 = widget_base(col_2, /column, xsize=140, /align_center )
base2 = widget_base(base1, /column, /frame, /align_center)
slice = widget_button(base2, xsize=120, ysize=35,  value='Extract slice', uvalue='slice', /align_center )

dumb = widget_base(col_2, /column, xsize=5)

base1 = widget_base(col_2, /column, xsize=140, /align_center )
base2 = widget_base(base1, /column, /frame, /align_center)
psfile = widget_button(base2, xsize=120, ysize=35,  value='Write Postscript', uvalue='ps', /align_center )


;---------------------- Start GUI
widget_control, main, /realize
widget_control, win0, get_value = windexm
widget_control, win1, get_value = windex1
widget_control, win2, get_value = windex2

; Update values
s.windex1 = windex1
s.windex2 = windex2
s.hdr_coor = hdr_coor

wset, windexm
tvimage, logo, /keep, /erase, xsize=xlogo, ys=ylogo, background=255


;---------------------- Execute GUI, set structure values

widget_control, main, set_uvalue=s
xmanager, 'view_ifs', main, /no_block


;---------------------- Display image first time (original view_3D)

loadct, 1, /silent

; Spaxels window limits
box_limits = pingsoft_boxlim(id,x,y,radius)
xmax_mosaic = box_limits[0]
xmin_mosaic = box_limits[1]
ymax_mosaic = box_limits[2]
ymin_mosaic = box_limits[3]


; Spectral window
set_font
wset, windex1
plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[0, 10], xs=1, ys=1, $
      xthick=wparam.wthick, ythick=wparam.wthick, position=wparam.pos2, charsize=wparam.csize, $
      xtitle=wparam.xtitle2, ytitle=wparam.ytitle2

; mas ?
if mas eq 0 then begin
   xtitle_w0 = wparam.xtitle0
   ytitle_w0 = wparam.ytitle0
endif
if mas eq 1 then begin
   xtitle_w0 = wparam.xtitle3
   ytitle_w0 = wparam.ytitle3
endif


; FoV window 
wset, windex2
plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
      xs=1, ys=1, xthick=wparam.wthick, ythick=wparam.wthick, position=wparam.pos0, charsize=wparam.csize, $
      xtitle=xtitle_w0, ytitle=ytitle_w0, /noerase


;--------- Plot spaxels

if not keyword_set(ct) then ct = 1

for i=0l, n_elements(x)-1 do begin

    xcoor =  x[i]
    ycoor =  y[i]

    x_fib = xarr + xcoor
    y_fib = yarr + ycoor

    loadct, ct, file=ctfile, /silent
    polyfill, x_fib, y_fib, color=scale[i]

    loadct, 1, /silent

 endfor

print, ''
print, ''
print, ' PINGSoft: IFS viewer'
print, ' =============================================='
print, ''
print, ' Filter:  '+sband
print, ''
print, ' Move mouse over the mosaic to plot the spectra'
print, ''
print, '  LEFT-click: prints spaxel information on terminal'
print, ''
print, '  RIGHT-click: access widget options'
print, ''
print, ''

plot_gui, s, skip=1

end

